package junit.log4j; // Generated package name
/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.<p>
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.<p>
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA<br>
 * http://www.gnu.org/copyleft/lesser.html
 */

import org.apache.log4j.Appender;
import org.apache.log4j.Layout;
import org.apache.log4j.spi.ErrorHandler;
import org.apache.log4j.spi.Filter;
import org.apache.log4j.spi.LoggingEvent;
import org.apache.log4j.spi.OptionHandler;

// Configuration Management Information: 
// -------------------------------------
// $Id: JUnitAppender.java,v 1.2 2001/11/12 21:36:14 wreissen Exp $
//
// Version History:
// ----------------
// $Log: JUnitAppender.java,v $
// Revision 1.2  2001/11/12 21:36:14  wreissen
// javadoc bug fixes
//
// Revision 1.1  2001/11/12 17:57:15  wreissen
// initial version.
//
//
// ***********************************************************************************
/**
 * <p>{@link org.apache.log4j.Appender Appender} that
 * joins <a href="http://www.junit.org">JUnit</a> and
 * <a href="http://jakarta.apache.org/log4j">Log4J</a>.</p>
 *
 *
 * Created: Mon Nov 12 07:49:22 2001
 *
 * @author <a href="mailto:wolfgang@openfuture.de">Wolfgang Reissenberger</a>
 * @version $Revision: 1.2 $
 */

public class JUnitAppender implements Appender, OptionHandler {
    
    private ErrorHandler errorHandler;
    private Layout layout;
    private String name;

    /** 
     * The first filter in the filter chain. Set to <code>null</code>
     * initially.
     */
    protected Filter headFilter;
    /** 
     * The last filter in the filter chain.
     */
    protected Filter tailFilter;


    /**
     * Creates a new <code>JUnitAppender</code> instance.
     *
     */
    public JUnitAppender() {
	super();
    }


    // implementation of org.apache.log4j.Appender interface

    
    /**
     * Get the value of name.
     * @return value of name.
     */
    public String getName() {
	return name;
    }
    
    /**
     * Set the value of name.
     * @param v  Value to assign to name.
     */
    public void setName(String  v) {
	this.name = v;
    }
    
    /**
     * Release any resources allocated within the appender.
     * Currently does nothing.
     *
     */
    public void close() {
    }

    /**
     * Add a filter to end of the filter list.
     *
     * @param newFilter new filter to be added
     */
    public synchronized void addFilter(Filter newFilter) {
	if(headFilter == null) {
	    headFilter = tailFilter = newFilter;
	} else {
	    tailFilter.next = newFilter;
	    tailFilter = newFilter;    
	}
    }

    /**
     * Returns the head Filter.
     *
     * @return the head Filter
     */
    public synchronized Filter getFilter() {
	return headFilter;
    }

    /**
     * Clear the filters chain.
     *
     */
    public synchronized void clearFilters() {
	headFilter = tailFilter = null;
    }


    /**
     * {@link org.apache.log4j.spi.Filter#decide(org.apache.log4j.spi.LoggingEvent)
     *        Filter} the event using the current {@link #getFilter() filters}. If
     * there exists an {@link org.apache.log4j.spi.Filter#ACCEPT accepting} filter
     * or none in the sequence {@link org.apache.log4j.spi.Filter#DENY denies} the
     * event, it is {@link #append(org.apache.log4j.spi.LoggingEvent) appended}.
     *
     * @param event a value of type 'LoggingEvent'
     */
    public synchronized void doAppend(LoggingEvent event) {

	Filter f = this.headFilter;
    
	while(f != null && f.decide(event) != Filter.ACCEPT) {

	    if (f.decide(event) == Filter.DENY) return;

	    f = f.next;
	}
	this.append(event);    
    }

    
    /**
     * Get the value of errorHandler.
     * @return value of errorHandler.
     */
    public synchronized ErrorHandler getErrorHandler() {
	return errorHandler;
    }
    
    /**
     * Set the value of errorHandler.
     * @param v  Value to assign to errorHandler.
     */
    public synchronized void setErrorHandler(ErrorHandler  v) {
	this.errorHandler = v;
    }
    

    /**
     * Get the value of layout.
     * @return value of layout.
     */
    public synchronized Layout getLayout() {
	return JUnitAppenderSingleton.getInstance().getLayout();
    }
    
    /**
     * Set the value of layout.
     * @param v  Value to assign to layout.
     */
    public synchronized void setLayout(Layout  v) {
	JUnitAppenderSingleton.getInstance().setLayout(v);
    }


    /**
     * Yes, this appender requires a layout.
     *
     * @return <code>true</code>
     */
    public boolean requiresLayout() {
	return true;
    }
    
    /**
     * Sends the event to the
     * {@link JUnitAppenderSingleton#addLoggingEvent(org.apache.log4j.spi.LoggingEvent)
     *        connector}.
     * @param event a <code>LoggingEvent</code> value
     * @see #getLayout()
     */
    protected synchronized void append(LoggingEvent event) {
	JUnitAppenderSingleton.getInstance().addLoggingEvent(event);
    }


    // implementation of org.apache.log4j.spi.OptionHandler interface
    
    /**
     * Does nothing, since this class has no specific options.
     */
    public void activateOptions() {
    }

    /**
     * Does nothing, since this class has no specific options.
     * @deprecated Use the getter method for the option directly instead
     * @return empty array
     */
    public String[] getOptionStrings() {
	return (new String[0]);
    }

    /**
     * Does nothing, since this class has no specific options.
     * @deprecated Use the setter method for the option directly instead
     *
     * @param key option key
     * @param value option value
     */
    public void setOption(String key, String value) {
    }
    
} // JUnitAppender
